package com.sun.xml.internal.bind.v2.runtime;

import javax.xml.bind.helpers.ValidationEventImpl;
import javax.xml.stream.XMLStreamException;
import java.io.IOException;
import java.lang.reflect.Array;
import javax.xml.bind.JAXBException;
import org.xml.sax.SAXException;
import java.util.List;
import com.sun.xml.internal.bind.api.AccessorException;
import com.sun.xml.internal.bind.WhiteSpaceProcessor;
import com.sun.istack.internal.FinalArrayList;
import com.sun.xml.internal.bind.v2.runtime.unmarshaller.UnmarshallingContext;
import com.sun.xml.internal.bind.v2.runtime.unmarshaller.Loader;

final class ValueListBeanInfoImpl extends JaxBeanInfo
{
    private final Class itemType;
    private final Transducer xducer;
    private final Loader loader;
    
    public ValueListBeanInfoImpl(JAXBContextImpl owner, Class arrayType) throws JAXBException {
        super(owner, null, arrayType, false, true, false);
        this.loader = new Loader(true) {
            public void text(UnmarshallingContext.State state, CharSequence text) throws SAXException {
                List<Object> r = new FinalArrayList();
                int idx = 0;
                int len = text.length();
                while (true) {
                    int p;
                    for (p = idx; p < len && !WhiteSpaceProcessor.isWhiteSpace(text.charAt(p)); ++p) {}
                    CharSequence token = text.subSequence(idx, p);
                    if (!token.equals("")) {
                        try {
                            r.add(ValueListBeanInfoImpl.this.xducer.parse(token));
                        }
                        catch (AccessorException e) {
                            handleGenericException(e, true);
                            continue;
                        }
                    }
                    if (p == len) {
                        break;
                    }
                    while (p < len && WhiteSpaceProcessor.isWhiteSpace(text.charAt(p))) {
                        ++p;
                    }
                    if (p == len) {
                        break;
                    }
                    idx = p;
                }
                state.setTarget(ValueListBeanInfoImpl.access$100(ValueListBeanInfoImpl.this, r));
            }
        };
        this.itemType = this.jaxbType.getComponentType();
        this.xducer = owner.getBeanInfo(arrayType.getComponentType(), true).getTransducer();
        assert this.xducer != null;
        return;
    }
    
    private Object toArray(List list) {
        int len = list.size();
        Object array = Array.newInstance(this.itemType, len);
        for (int i = 0; i < len; ++i) {
            Array.set(array, i, list.get(i));
        }
        return array;
    }
    
    @Override
    public void serializeBody(Object array, XMLSerializer target) throws SAXException, IOException, XMLStreamException {
        for (int len = Array.getLength(array), i = 0; i < len; ++i) {
            Object item = Array.get(array, i);
            try {
                this.xducer.writeText(target, item, "arrayItem");
            }
            catch (AccessorException e) {
                target.reportError("arrayItem", e);
            }
        }
    }
    
    @Override
    public final void serializeURIs(Object array, XMLSerializer target) throws SAXException {
        if (this.xducer.useNamespace()) {
            for (int len = Array.getLength(array), i = 0; i < len; ++i) {
                Object item = Array.get(array, i);
                try {
                    this.xducer.declareNamespace(item, target);
                }
                catch (AccessorException e) {
                    target.reportError("arrayItem", e);
                }
            }
        }
    }
    
    @Override
    public final String getElementNamespaceURI(Object array) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public final String getElementLocalName(Object array) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public final Object createInstance(UnmarshallingContext context) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public final boolean reset(Object array, UnmarshallingContext context) {
        return false;
    }
    
    @Override
    public final String getId(Object array, XMLSerializer target) {
        return null;
    }
    
    @Override
    public final void serializeAttributes(Object array, XMLSerializer target) {
    }
    
    @Override
    public final void serializeRoot(Object array, XMLSerializer target) throws SAXException {
        target.reportError(new ValidationEventImpl(1, Messages.UNABLE_TO_MARSHAL_NON_ELEMENT.format(new Object[] { array.getClass().getName() }), null, null));
    }
    
    @Override
    public final Transducer getTransducer() {
        return null;
    }
    
    @Override
    public final Loader getLoader(JAXBContextImpl context, boolean typeSubstitutionCapable) {
        return this.loader;
    }
    
    static Object access$100(ValueListBeanInfoImpl x0, List x1) {
        return x0.toArray(x1);
    }
}
